package com.jiuqi.util;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;

public abstract class Stream {
   private boolean encode = true;
   private String charset = "GBK";
   private static final int BUFF_SIZE = 1024;
   public static final int SEEK_ORIGIN_START = 0;
   public static final int SEEK_ORIGIN_CURRENT = 1;
   public static final int SEEK_ORIGIN_END = 2;

   public void setUseEncode(boolean useEncode) {
      this.encode = useEncode;
   }

   public boolean getUseEncode() {
      return this.encode;
   }

   public void setCharset(String encodeCharset) {
      if (encodeCharset != null && encodeCharset.length() != 0) {
         this.charset = encodeCharset;
      }

   }

   public String getCharset() {
      return this.charset;
   }

   public abstract void setSize(long var1) throws StreamException;

   public abstract int read(byte[] var1, int var2, int var3) throws StreamException;

   public abstract byte read() throws StreamException;

   public abstract int write(byte[] var1, int var2, int var3) throws StreamException;

   public abstract void write(byte var1) throws StreamException;

   public abstract long seek(long var1, int var3) throws StreamException;

   public void readBuffer(byte[] buffer, int pos, int count) throws StreamException {
      if (count != 0 && this.read(buffer, pos, count) != count) {
         throw new StreamException("Stream read error");
      }
   }

   public void writeBuffer(byte[] buffer, int pos, int count) throws StreamException {
      if (count != 0 && this.write(buffer, pos, count) != count) {
         throw new StreamException("Stream write error");
      }
   }

   public long copyFrom(Stream source, long count) throws StreamException {
      if (source == null) {
         return 0L;
      } else {
         if (count == 0L) {
            source.setPosition(0L);
            count = source.getSize();
         }

         int bufSize;
         if (count > 1024L) {
            bufSize = 1024;
         } else {
            bufSize = (int)count;
         }

         int n;
         for(byte[] buffer = new byte[1024]; count != 0L; count -= (long)n) {
            if (count > (long)bufSize) {
               n = bufSize;
            } else {
               n = (int)count;
            }

            source.readBuffer(buffer, 0, n);
            this.writeBuffer(buffer, 0, n);
         }

         return 0L;
      }
   }

   public long copyFrom(InputStream stream, long size) throws StreamException, IOException {
      if (stream == null) {
         return 0L;
      } else {
         long readcount = 0L;
         byte[] buf;
         int b;
         if (size == 0L) {
            buf = new byte[1024];
            int b = false;

            do {
               b = stream.read(buf);
               if (b > 0) {
                  this.write(buf, 0, b);
                  readcount += (long)b;
               }
            } while(b > 0);

            while(b > 0) {
               b = stream.read(buf);
               if (b > 0) {
                  this.write(buf, 0, b);
               }
            }
         } else {
            buf = new byte[1024];

            do {
               int blockSize = size > 1024L ? 1024 : (int)size;
               b = stream.read(buf, 0, blockSize);
               this.write(buf, 0, b);
               readcount += (long)b;
               size -= (long)b;
            } while(b > 0 && size > 0L);
         }

         return readcount;
      }
   }

   public long copyTo(OutputStream stream, long size) throws StreamException, IOException {
      long writecount = 0L;
      byte[] buffer = new byte[1024];

      for(int writesize = this.read(buffer, 0, 1024); writesize > 0 && size > 0L; size -= (long)writesize) {
         stream.write(buffer, 0, writesize);
         writesize = this.read(buffer, 0, 1024);
         writecount += (long)writesize;
      }

      return writecount;
   }

   public byte[] getBytes() throws StreamException {
      long oldPos = this.getPosition();

      byte[] var5;
      try {
         byte[] rt = new byte[(int)this.getSize()];
         this.setPosition(0L);
         this.read(rt, 0, rt.length);
         var5 = rt;
      } finally {
         this.setPosition(oldPos);
      }

      return var5;
   }

   public long getSize() throws StreamException {
      long pos = this.seek(0L, 1);
      long r = this.seek(0L, 2);
      this.seek(pos, 0);
      return r;
   }

   public long getPosition() throws StreamException {
      return this.seek(0L, 1);
   }

   public void setPosition(long pos) throws StreamException {
      this.seek(pos, 0);
   }

   public short readShort() throws StreamException {
      int b1 = this.read() & 255;
      int b2 = this.read() & 255;
      return (short)((b2 << 8) + (b1 << 0));
   }

   public int readWord() throws StreamException {
      int b1 = this.read() & 255;
      int b2 = this.read() & 255;
      return (b2 << 8) + (b1 << 0);
   }

   public int readInt() throws StreamException {
      int b1 = this.read() & 255;
      int b2 = this.read() & 255;
      int b3 = this.read() & 255;
      int b4 = this.read() & 255;
      return (b4 << 24) + (b3 << 16) + (b2 << 8) + (b1 << 0);
   }

   public float readFloat() throws StreamException {
      return Float.intBitsToFloat(this.readInt());
   }

   public double readDouble() throws StreamException {
      return Double.longBitsToDouble(this.readLong());
   }

   public String readString(int length) throws StreamException {
      byte[] str = new byte[length];
      this.read(str, 0, length);
      return this.decodeString(str);
   }

   public String decodeString(byte[] data) {
      if (this.encode) {
         try {
            return new String(data, this.charset);
         } catch (UnsupportedEncodingException var2) {
            return new String(data);
         }
      } else {
         return new String(data);
      }
   }

   public String readStringBySize() throws StreamException {
      int len = this.readInt();
      return len == 0 ? null : this.readString(len);
   }

   public boolean readBool() throws StreamException {
      return this.read() > 0;
   }

   public long readLong() throws StreamException {
      long b1 = (long)(this.read() & 255);
      long b2 = (long)(this.read() & 255);
      long b3 = (long)(this.read() & 255);
      long b4 = (long)(this.read() & 255);
      long b5 = (long)(this.read() & 255);
      long b6 = (long)(this.read() & 255);
      long b7 = (long)(this.read() & 255);
      long b8 = (long)this.read();
      return (b8 << 56) + (b7 << 48) + (b6 << 40) + (b5 << 32) + (b4 << 24) + (b3 << 16) + (b2 << 8) + (b1 << 0);
   }

   public void writeShort(short value) throws StreamException {
      this.write((byte)(value >>> 0 & 255));
      this.write((byte)(value >>> 8 & 255));
   }

   public void writeWord(int value) throws StreamException {
      this.write((byte)(value >>> 0 & 255));
      this.write((byte)(value >>> 8 & 255));
   }

   public void writeInt(int value) throws StreamException {
      this.write((byte)(value >>> 0 & 255));
      this.write((byte)(value >>> 8 & 255));
      this.write((byte)(value >>> 16 & 255));
      this.write((byte)(value >>> 24 & 255));
   }

   public void writeLong(long value) throws StreamException {
      this.write((byte)((int)(value >>> 0 & 255L)));
      this.write((byte)((int)(value >>> 8 & 255L)));
      this.write((byte)((int)(value >>> 16 & 255L)));
      this.write((byte)((int)(value >>> 24 & 255L)));
      this.write((byte)((int)(value >>> 32 & 255L)));
      this.write((byte)((int)(value >>> 40 & 255L)));
      this.write((byte)((int)(value >>> 48 & 255L)));
      this.write((byte)((int)(value >>> 56 & 255L)));
   }

   public void writeFloat(float value) throws StreamException {
      this.writeInt(Float.floatToIntBits(value));
   }

   public void writeDouble(double value) throws StreamException {
      this.writeLong(Double.doubleToLongBits(value));
   }

   public void writeBool(boolean value) throws StreamException {
      byte b = value ? 1 : 0;
      this.write((byte)b);
   }

   public void writeString(String value) throws StreamException {
      byte[] b = this.encodeString(value);
      this.write(b, 0, b.length);
   }

   public byte[] encodeString(String value) {
      if (value == null) {
         return null;
      } else if (this.encode) {
         try {
            return value.getBytes(this.charset);
         } catch (UnsupportedEncodingException var2) {
            return value.getBytes();
         }
      } else {
         return value.getBytes();
      }
   }

   public void writeStringWithSize(String value) throws StreamException {
      if (value == null) {
         this.writeInt(0);
      } else {
         byte[] b = this.encodeString(value);
         this.writeInt(b.length);
         this.write(b, 0, b.length);
      }
   }

   public void loadFromFile(String fileName) throws StreamException, IOException {
      InputStream source = new FileInputStream(fileName);

      try {
         this.loadFromStream(source);
      } finally {
         source.close();
      }

   }

   public void loadFromStream(InputStream inStream) throws StreamException, IOException {
      this.copyFrom(inStream, 0L);
   }

   public void saveToFile(String fileName) throws StreamException, IOException {
      OutputStream dest = new FileOutputStream(fileName);

      try {
         this.saveToStream(dest);
      } finally {
         dest.close();
      }

   }

   public void saveToStream(OutputStream outStream) throws StreamException, IOException {
      this.setPosition(0L);
      this.copyTo(outStream, this.getSize());
   }
}
